#ifndef Vector2d_H
#define Vector2d_H


class Vector2d
{
public:
	// Constructors and Destructors
	Vector2d(void) {} 

	Vector2d( const float fx, const float fy )
	{
		x = fx ;
		y = fy ;
	}

	// use default destructor
	
	// Assignment operators
	inline void	Vector2d::Set( const Vector2d &v );
	inline void	Vector2d::Set( const float fX, const float fY );

	// Arithmetic operators
	inline Vector2d operator -  ( void ) const;					// Unary operator
	inline void     operator += ( const Vector2d& v );			// increment by a Vector2d	
	inline void	    operator -= ( const Vector2d& v );			// decrement by a Vector2d
	inline void     operator *= ( const Vector2d& v );			// component-wise multiplication  
	inline void     operator /= ( const Vector2d& v );			// component-wise division  
	inline void     operator *= ( const float fScale );			// multiplication by a constant
	inline void     operator /= ( const float fScale );			// division by a constant
	inline Vector2d operator +  ( const Vector2d& v ) const;
	inline Vector2d operator -  ( const Vector2d& v ) const;
	inline Vector2d operator *  ( const Vector2d& v ) const;
	inline Vector2d operator /  ( const Vector2d& v ) const;
	inline Vector2d operator *  ( const float fScale ) const;
	inline Vector2d operator /  ( const float fScale ) const;

	// Vector specific operators
	inline float	Dot( const Vector2d &v ) const ;
	float	Len(void) const ;
	float	SquaredLen(void) const; 

	// Special functions
	inline Vector2d&	Normalize() ;
	inline float		NormalizeLength( const Vector2d &v ) ;
	inline void			Lerp( const Vector2d &ptFrom, const Vector2d &ptTo, float t ) ;

	// "friends"
	friend inline Vector2d operator * ( const float f, const Vector2d &v );
//	friend bool Equals( const Vector2d &v1, const Vector2d &v2, const float fEpsilon = FLOAT_EPSILON );
	friend float Dot( const Vector2d &v1, const Vector2d &v2 );

protected:

private:
	// Private copy constructor and assignment operator undefined.
	//Vector2d( const  Vector2d& obj );
	//Vector2d& operator=( const  Vector2d & obj );

	// Data members

public:
	float x, y;
};

typedef Vector2d Point2d ;

inline void	Vector2d::Set( const Vector2d &v )
{
	x = v.x ;
	y = v.y ;
}

inline void	Vector2d::Set( const float fX, const float fY )
{
	x = fX ;
	y = fY ;
}

inline Vector2d	Vector2d::operator - ( void ) const
{
	return Vector2d( -x, -y ) ;
}

inline void Vector2d::operator += ( const Vector2d &v )
{
	x += v.x ;
	y += v.y ;
}

inline void Vector2d::operator -= ( const Vector2d& v )
{
   x -= v.x ;
   y -= v.y ;
}

inline void Vector2d::operator*= ( const Vector2d &v )
{
	x *= v.x ;
	y *= v.y ;
}

inline void Vector2d::operator /= ( const Vector2d &v )
{
	x /= v.x ;
	y /= v.y ;
}

inline void Vector2d::operator *= ( const float fScale )
{
	x *= fScale ;
	y *= fScale ;
}

inline void Vector2d::operator /= ( const float fScale )
{
	x /= fScale ;
	y /= fScale ;
}

inline Vector2d Vector2d::operator + ( const Vector2d &v ) const
{
	return Vector2d( x + v.x, y + v.y );
}

inline Vector2d Vector2d::operator - ( const Vector2d &v ) const
{
	return Vector2d( x - v.x, y - v.y );
}

inline Vector2d Vector2d::operator * ( const Vector2d& v ) const
{
	return Vector2d( x * v.x, y * v.y );
}

inline Vector2d Vector2d::operator / ( const Vector2d& v ) const
{
	return Vector2d( x / v.x, y / v.y );
}

inline Vector2d Vector2d::operator * ( const float fScale ) const 
{
	return Vector2d( fScale * x, fScale * y );
}

inline Vector2d Vector2d::operator / ( const float fScale ) const 
{
	return Vector2d( x / fScale, y / fScale ) ;
}

inline float Vector2d::Dot( const Vector2d &v ) const
{
	return ( x * v.x  +  y * v.y  ) ;
}

inline float Vector2d::Len(void) const 
{
	return fsqrt( x * x  +  y * y ) ;
}

inline float Vector2d::SquaredLen(void) const 
{
	return ( x * x  +  y * y  ) ;
}

inline Vector2d& Vector2d::Normalize(void)
{
	float fLen = Len() ;
	
	float fReciprocal ;
	if ( fLen > .0001 )
	{
		fReciprocal = 1.0f / fLen ;
	}
	else
	{
		//WARNING: Can't normalize a zero-length vector
		fReciprocal = 0.0f ; 
	}

	x *= fReciprocal ;
	y *= fReciprocal ;

	return *this ;
}


inline float Vector2d::NormalizeLength( const Vector2d &v )
{
	float fLen = v.Len() ;
	
	float fReciprocal ;
	if ( fLen > .0001 )
	{
		fReciprocal = 1.f / fLen ;
	}
	else
	{
		//WARNING: Can't normalize a zero-length vector
		fReciprocal = 0.0f ; 
	}

	x = v.x * fReciprocal ;
	y = v.y * fReciprocal ;

	return fLen;
}

inline void Vector2d::Lerp( const Vector2d &ptFrom, const Vector2d &ptTo, float t )
{
	*this = ptFrom + (ptTo - ptFrom)*t ;
}

//	"friend" functions

inline Vector2d operator * ( const float f, const Vector2d &v )
{
	return Vector2d( f * v.x, f * v.y ) ;
}



inline float Dot( const Vector2d &v1, const Vector2d &v2 )
{
	return v1.Dot(v2);
}

	
inline bool operator == ( const Vector2d &vA, const Vector2d &vB ) 
{
	return ( vA.x == vB.x  &&  vA.y == vB.y ) ;	
}
	
inline bool operator != ( const Vector2d &vA, const Vector2d &vB ) 
{
	return ( vA.x != vB.x  ||  vA.y != vB.y ) ;	
}
	

#endif  //Vector2d_H

